<?php

namespace Teknisa\EST\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Doctrine\ORM\EntityManager;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;


class Cch12000ExportaImportaSAPController extends \Teknisa\EST\Controllers\TeknisaCrud
{

    public function __construct($dataSourceManager, $enviroment, $entityManager)
    {
        $this->dataSourceManager = $dataSourceManager;
        $this->environment = $enviroment;
        $this->entityManager = $entityManager;
    }

    public function exportOrImport(DTO\Request $request, DTO\Response $response)
    {
        try {
  
            $this->response = $response;
            $row = $request->getRow();
    
            $row = $this->verificaCodigoFilial($row);

            switch ($row['FILE_TYPE']) {
                case '01':
                    $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CCH_EXPORTA_PRODUTO_SAP');
                     
                    return $this->executeProcedure($procedure);
                  
                    break;
                    
                case '02':
                    $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CCH_IMPORTA_FILIAL_SAP');
                    
                    return $this->executeProcedure($procedure);
                  
                    break;
                    
                case '03':
                    $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CCH_IMPORTA_FORNECEDOR_SAP');
                     
                    return $this->executeProcedure($procedure);
                  
                    break;
                    
                case '04':
                    $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CCH_IMPORTA_CLIENTE_SAP');
                    
                    return $this->executeProcedure($procedure);
                  
                    break;
                    
                    
                case '05':
                    $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CCH_IMPORTA_CLIENFILIAL_SAP');
                    
                    return $this->executeProcedure($procedure);
                  
                    break;
                    
                case '06':
                    $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CCH_EXPORTA_NF_ENTRADA_SAP');
                    
                    $this->addParams($procedure, $row, true);
                    
                    break;

                case '07':
                    $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CCH_EXPORTA_NF_SAIDA_SAP');
                    
                    $this->addParams($procedure, $row, true);
                    
                    break;

                case '08':
                    $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CCH_EXPORTA_TI_SAP');
                    
                    //isso est´aqui e não no "addParams" poruqe essa procedure tem menos parametros e uma ordem diferente
                    
                    $pTcdFilial = $row['P_TCDFILIAL'] ?? null;
                    $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row['P_CDFILIAL'], Param::PARAM_TYPE_STR));
                    $procedure->addParam(new Param('P_DATAFIN', Param::PARAM_INPUT, $row['PERIODO']['end'], Param::PARAM_TYPE_STR)); 
                    $procedure->addParam(new Param('P_TCDFILIAL', Param::PARAM_INPUT, $pTcdFilial, Param::PARAM_TYPE_STR));
                    
                    return $this->executeProcedure($procedure);
                    
                    break;

            }

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    private function verificaCodigoFilial($row)
    {
        if (count($row['P_CDFILIAL']) == 0 || empty($row['P_CDFILIAL'])) {
            $row['P_TCDFILIAL'] = 'T';
        }

        if (count($row['P_CDFILIAL']) > 0) {
            $row['P_CDFILIAL'] = '_' . implode('_', $row['P_CDFILIAL']) . '_';
        }
        
        return $row;
    }
    
    private function executeProcedure($procedure){
        try{
            $procedure->execute();
           
            return $this->response->addDataSet(new DataSet('message', ['Operação executada com sucesso.']));
           
        }catch (\Exception $e) {
            $this->response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    private function addParams($procedure, $row, $addRENumber = false){
        $pTcdFilial = $row['P_TCDFILIAL'] ?? null;
      
        if($addRENumber){      
            $procedure->addParam(new Param('P_DATAINI', Param::PARAM_INPUT, $row['PERIODO']['start'], Param::PARAM_TYPE_STR)); 
            $procedure->addParam(new Param('P_DATAFIN', Param::PARAM_INPUT, $row['PERIODO']['end'], Param::PARAM_TYPE_STR)); 
            $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row['P_CDFILIAL'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_TCDFILIAL', Param::PARAM_INPUT, $pTcdFilial, Param::PARAM_TYPE_STR));
        }
        
        if(!$addRENumber){      
            $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row['P_CDFILIAL'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DATAINI', Param::PARAM_INPUT, $row['PERIODO']['start'], Param::PARAM_TYPE_STR)); 
            $procedure->addParam(new Param('P_DATAFIN', Param::PARAM_INPUT, $row['PERIODO']['end'], Param::PARAM_TYPE_STR)); 
            $procedure->addParam(new Param('P_TCDFILIAL', Param::PARAM_INPUT, $pTcdFilial, Param::PARAM_TYPE_STR));
        }
       
        return $this->executeProcedure($procedure);
    }
    
}
