<?php
namespace Teknisa\EST\Modules\Lancamentos\Controllers;

use Teknisa\EST\Services\ValidacoesEstoqueService;
use Teknisa\EST\Controllers\TeknisaCrud;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;

class Est11000Entrada extends TeknisaCrud {

    protected $dataSourceName = "lanctoestoq_entrada";
    protected $parameterBagColumns = ["DTFINAL",'DTINICIAL','DATA', 'INTERVALO'];

    protected $Est11000Service;
    protected $validacoesEstoqueService;

    protected $connection;

    public function __construct($crudService, $parameterBag, $Est11000Service, ValidacoesEstoqueService $validacoesEstoqueService, $entityManager, $enviroment) {
        parent::__construct($crudService,$parameterBag);

        $this->Est11000Service = $Est11000Service;
        $this->validacoesEstoqueService = $validacoesEstoqueService;

        $this->connection = $entityManager->getConnection();
        
        $this->NRORG = $enviroment->getNrOrg();
    }

    public function saveEntrada(DTO\Request $request, DTO\Response $response) {

    }

    public function save(DTO\Request $request, DTO\Response $response) {
        try {
            $this->dataSourceName = 'lanctoestoq';
            
            parent::save($request,$response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function delete(DTO\Request $request, DTO\Response $response) {
        $retirada = $request->getRow();
        $lancamento = $request->getRow();
        $this->connection->beginTransaction();
        try {
            $this->Est11000Service->verificaInventarioRotativo($lancamento['CDFILIAL'],
                                                              $lancamento['DTLANCESTQ']);

            $gambiarra = $this->validacoesEstoqueService->buscaLanctoestoq($lancamento["CDFILIAL"], $lancamento["NRLANCESTQ"], $this->NRORG);
            
            $this->Est11000Service->processDeleteEntrada($retirada);

            $this->validacoesEstoqueService->atualizaEstoque($lancamento["CDFILIAL"], $lancamento["NRLANCESTQ"]);

            $this->connection->commit();
        } catch(\Exception $e) {
            $this->connection->rollBack();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function findPosestdia(DTO\Request\Filter $request, DTO\Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $filter = [];
        foreach ($conditions as $condition) {
            $filter[$condition["columnName"]] = $condition["value"];
        }

        try {
            $data = $this->Est11000Service->getPosestDia($filter);

            $dataSet = new DataSet('posestdia', array($data));
            $response->addDataSet($dataSet);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}